#ifndef _MESHTOPOLOGYOFFSETTER_H_
#define _MESHTOPOLOGYOFFSETTER_H_
/*******************************************************************************
  Copyright (c) 2002-2007 J. P. Abelanet/Basis Software.
  All Rights Reserved.

  -> THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF BASIS SOFTWARE. <-

  FILE
    MeshTopologyOffsetter.h
    
  DESCRIPTION
    Class for offsetting surface meshes.
    
  AUTHOR
    J. P. Abelanet - 05/22/02
*******************************************************************************/

/* library includes */
#include <vmath/Array.h>
#include <vmath/Base.h>
#include <vmath/Error.h>
#include <vmath/Vector3D.h>



namespace GE {

/* forward declarations */
template <class Key, class T> class Hash;
class Entity;
class MeshModel;



/*******************************************************************************
  ABSTRACT
    Class for offsetting surface meshes.
    
  AUTHOR
    J. P. Abelanet - 05/22/02
*******************************************************************************/
class MeshTopologyOffsetter {

private:	// private constructor - NOT IMPLEMENTED!
  MeshTopologyOffsetter ();
  
public:	// public classes

/*******************************************************************************
  ABSTRACT
    This class contains information about localized errors with offsets.
      
  AUTHOR
    J. P. Abelanet - 09/16/05
*******************************************************************************/
  struct LocalizedError {
    Vector3D	OriginalPoint;		/* Point before deformation attempted		*/
    Vector3D	AttemptedLocation;	/* Location that OriginalPoint tried to move to	*/
  };

public:	// public static methods
  static Error MeshModel_Create_Offset (const MeshModel *model, const Real64 Offset,
       const Real64 deflectionBuffer, MeshModel **const OffsetModel,
       Hash<Entity*, Entity*> **const entityHash,
       Array<LocalizedError> *const localizedErrors);
};


}	// namespace GE

#endif /* !_MESHTOPOLOGYOFFSETTER_H_ */
